job_user = "ravisharma@scoredata.com"
job_password = "ravi@score1"

config_url = 'http://develop.scoredata.com/dw/client/config'
process_log_url = "http://develop.scoredata.com/dw/datatransfer/process_log/"
