from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='scorefastlib',
    version='0.1.3',
    license='',
    author='scorefast',
    author_email='ravisharma@scoredata.com',
    description='scorefast customer tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ScoreDataCorporation/scorefast-lib",
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    packages=find_packages(),
)
